@@ -77,25 +77,18 @@ select_vars_ <- function(vars, args, include = character(), exclude = character(
   ind_list <- lazyeval::lazy_eval(args, c(names_list, select_funs))
   names(ind_list) <- names2(args)
 
-  ind <- unlist(ind_list)
-  incl <- ind[ind > 0]
+  is_numeric <- vapply(ind_list, is.numeric, logical(1))
+  if (any(!is_numeric)) {
+    bad_inputs <- lapply(args[!is_numeric], `[[`, "expr")
+    labels <- vapply(bad_inputs, deparse_trunc, character(1))
 
-  # If only negative values, implicitly assumes all variables to be removed.
-  if (sum(ind > 0) == 0 && sum(ind < 0) > 0) {
-    incl <- seq_along(vars)
-  }
-  # Remove duplicates (unique loses names)
-  incl <- incl[!duplicated(incl)]
-
-  # Remove variables to be excluded (setdiff loses names)
-  excl <- abs(ind[ind < 0])
-  incl <- incl[match(incl, excl, 0L) == 0L]
-
-  bad_idx <- incl < 0 | incl > length(vars)
-  if (any(bad_idx)) {
-    stop("Bad indices: ", paste0(which(bad_idx), collapse = ", "),
+    stop("All select() inputs must resolve to integer column positions.\n",
+      "The following do not:\n", paste("* ", labels, collapse = "\n"),
       call. = FALSE)
   }
+
+  incl <- combine_vars(vars, ind_list)
+
   # Include/exclude specified variables
   sel <- setNames(vars[incl], names(incl))
   sel <- c(setdiff2(include, sel), sel)
@@ -128,7 +121,7 @@ rename_vars <- function(vars, ...) {
 #' @rdname select_vars
 rename_vars_ <- function(vars, args) {
   if (any(names2(args) == "")) {
-    stop("All arguments to rename must be named.", stop = FALSE)
+    stop("All arguments to rename must be named.", call. = FALSE)
   }
 
   args <- lazyeval::as.lazy_dots(args)
