@@ -13,6 +13,9 @@ namespace dplyr {
         virtual bool is_factor_collecter() const{ 
             return false ;    
         }
+        virtual bool is_logical_all_na() const {
+            return false ;    
+        }
         virtual std::string describe() const = 0 ;
     } ;
     
@@ -47,6 +50,10 @@ namespace dplyr {
             return vector_class<RTYPE>() ; 
         }
         
+        bool is_logical_all_na() const {
+            return RTYPE == LGLSXP && all(is_na(data)).is_true() ;    
+        }
+        
     protected:
         Vector<RTYPE> data ;
     } ;
@@ -162,7 +169,7 @@ namespace dplyr {
         }
         
         bool can_promote(SEXP x) const {
-            return TYPEOF(x) == REALSXP || Rf_inherits( x, "factor" ) ;    
+            return TYPEOF(x) == REALSXP ;    
         }
         
         std::string describe() const {
