@@ -8,7 +8,7 @@ namespace dplyr {
         typedef dplyr_hash_map<SEXP, SEXP> DataMap ;
 
         CallProxy( const Rcpp::Call& call_, LazySubsets& subsets_, const Environment& env_) :
-            call(call_), subsets(subsets_), proxies(), env(env_), hybrid(false)
+            call(call_), subsets(subsets_), proxies(), env(env_)
         {
             // fill proxies
             set_call(call);
@@ -16,49 +16,49 @@ namespace dplyr {
         }
 
         CallProxy( const Rcpp::Call& call_, const Rcpp::DataFrame& data_, const Environment& env_) :
-            call(call_), subsets(data_), proxies(), env(env_), hybrid(false)
+            call(call_), subsets(data_), proxies(), env(env_)
         {
             // fill proxies
             set_call(call);
         }
 
         CallProxy( const Rcpp::DataFrame& data_, const Environment& env_ ) :
-            subsets(data_), proxies(), env(env_), hybrid(false) {
+            subsets(data_), proxies(), env(env_){
         }
 
         CallProxy( const Rcpp::DataFrame& data_) :
-            subsets(data_), proxies(), hybrid(false) {
+            subsets(data_), proxies() {
         }
 
         ~CallProxy(){}
 
         SEXP eval(){
             if( TYPEOF(call) == LANGSXP ){
-                if(hybrid){
-                    return HybridCall(call,subsets,env).eval() ;
+                
+                if( can_simplify(call) ){
+                    SlicingIndex indices(0,subsets.nrows()) ;
+                    while(simplified(indices)) ;
+                    set_call(call) ;
                 }
+                
                 int n = proxies.size() ;
                 for( int i=0; i<n; i++){
                     proxies[i].set( subsets[proxies[i].symbol] ) ;
                 }
-
-                Shield<SEXP> res( call.eval(env) ) ;
-                return res ;
+                return call.eval(env) ;
             } else if( TYPEOF(call) == SYMSXP) {
                 // SYMSXP
                 if( subsets.count(call) ) return subsets.get_variable(call) ;
-                Shield<SEXP> res( call.eval(env) );
-                return res ;
+                return call.eval(env) ;
             }
             return call ;
         }
-
+        
         void set_call( SEXP call_ ){
             proxies.clear() ;
             call = call_ ;
-
+            
             if( TYPEOF(call) == LANGSXP ) traverse_call(call) ;
-            hybrid = can_simplify_call(call) ;
         }
 
         void input( Rcpp::String name, SEXP x ){
@@ -82,12 +82,48 @@ namespace dplyr {
         }
 
     private:
-
-        inline bool can_simplify_call( SEXP call_){
-            bool res =  can_simplify(call_);
-            return res ;
+        
+        bool simplified(const SlicingIndex& indices){
+            // initial
+            if( TYPEOF(call) == LANGSXP ){
+                Result* res = get_handler(call, subsets, env) ;
+                
+                if( res ){
+                    // replace the call by the result of process
+                    call = res->process(indices) ;
+                    
+                    // no need to go any further, we simplified the top level
+                    return true ;
+                }
+                
+                return replace( CDR(call), indices ) ;
+                
+            }
+            return false ;
         }
-
+        
+        bool replace( SEXP p, const SlicingIndex& indices ){
+            
+            SEXP obj = CAR(p) ;
+            
+            if( TYPEOF(obj) == LANGSXP ){
+                Result* res = get_handler(obj, subsets, env) ;
+                if(res){
+                    SETCAR(p, res->process(indices) ) ;
+                    return true ;
+                }
+                
+                if( replace( CDR(obj), indices ) ) return true ;   
+            }     
+            
+            if( TYPEOF(p) == LISTSXP ){
+                return replace( CDR(p), indices ) ;    
+            }
+              
+            return false ;
+        }
+        
+        
         void traverse_call( SEXP obj ){
             if( TYPEOF(obj) == LANGSXP && CAR(obj) == Rf_install("local") ) return ;
 
@@ -159,7 +195,7 @@ namespace dplyr {
         LazySubsets subsets ;
         std::vector<CallElementProxy> proxies ;
         Environment env;
-        bool hybrid ;
+        
     } ;
 
 }
