@@ -208,21 +208,27 @@ namespace dplyr{
     
     // -----------------
     inline void incompatible_join_visitor(SEXP left, SEXP right, const std::string& name_left, const std::string& name_right) {
-        std::stringstream s ;
-        s << "Can't join on '" 
-          << name_left 
-          << "' x '"
-          << name_right
-          << "' because of incompatible types (" 
-          << get_single_class(left) 
-          << "/" 
-          << get_single_class(right) 
-          << ")" ;
-        stop( s.str() ) ;    
+        stop( "Can't join on '%s' x '%s' because of incompatible types (%s / %s)", 
+            name_left, name_right, get_single_class(left), get_single_class(right) 
+        ) ;    
     }
     
-    JoinVisitor* join_visitor( SEXP left, SEXP right, const std::string& name_left, const std::string& name_right){
+    inline void warn( const char* msg ){
+        Rcpp::Function warning("warning") ;
+        warning( msg, _["call."] = false ) ;
+    }
+    
+    JoinVisitor* join_visitor( SEXP left, SEXP right, const std::string& name_left, const std::string& name_right, bool warn_ ){
         switch( TYPEOF(left) ){
+            case CPLXSXP:
+                {
+                    switch( TYPEOF(right) ){
+                    case CPLXSXP: return new JoinVisitorImpl<CPLXSXP, CPLXSXP>( left, right ) ;
+                    default:
+                        break ;
+                    }
+                    break ;
+                }
             case INTSXP:
                 {
                     bool lhs_factor = Rf_inherits( left, "factor" ) ;
@@ -231,7 +237,12 @@ namespace dplyr{
                             {
                                 bool rhs_factor = Rf_inherits( right, "factor" ) ;
                                 if( lhs_factor && rhs_factor){
-                                    return new JoinFactorFactorVisitor(left, right) ;
+                                    if( same_levels(left, right) ){
+                                        return new JoinFactorFactorVisitor_SameLevels(left, right) ;
+                                    } else {
+                                        if(warn_) warn( "joining factors with different levels, coercing to character vector" );
+                                        return new JoinFactorFactorVisitor(left, right) ;
+                                    }
                                 } else if( !lhs_factor && !rhs_factor) {
                                     return new JoinVisitorImpl<INTSXP, INTSXP>( left, right ) ;
                                 }
@@ -261,6 +272,7 @@ namespace dplyr{
                         case STRSXP:
                             {
                                 if( lhs_factor ){
+                                    if(warn_) warn( "joining factor and character vector, coercing into character vector" ) ;
                                     return new JoinFactorStringVisitor( left, right );     
                                 }
                             }
@@ -332,6 +344,7 @@ namespace dplyr{
                     case INTSXP:
                         {
                             if( Rf_inherits(right, "factor" ) ){
+                                if(warn_) warn( "joining character vector and factor, coercing into character vector" ) ;
                                 return new JoinStringFactorVisitor( left, right ) ;    
                             }
                             break ;
